
#This work is licensed under the Creative Commons Attribution 3.0 Unported License. To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/.

import kivy
kivy.require('1.7.1')
from kivy.uix.widget import Widget
from kivy.app import App
from kivy.uix.floatlayout import FloatLayout
from kivy.uix.gridlayout import GridLayout
from kivy.uix.image import Image
from kivy.lang import Builder
from kivy.animation import Animation, AnimationTransition
from kivy.properties import NumericProperty, ReferenceListProperty,\
	ObjectProperty, ListProperty
from kivy.clock import Clock
from kivy.vector import Vector
from kivy.core.window import Window
from random import randint
from kivy.core.audio import SoundLoader
from kivy.uix.screenmanager import ScreenManager, Screen, FadeTransition
import sys
from peri import PeriBoard
temp = 0.0

class Player(Widget):
	pass

class Arrowimg(Image):
	def move(self):
		self.x = self.parent.arrow.x
		self.center_y = self.parent.arrow.center_y

class Arrow(Image):
	stat = NumericProperty(0)
	velocity_x = NumericProperty(0)
	velocity_y = NumericProperty(0)
	velocity = ReferenceListProperty(velocity_x, velocity_y)

	def serve(self, vel=(0, 0)):
		self.velocity = vel

	def move(self):
		self.pos = Vector(*self.velocity) + self.pos

	def resetpos(self):
		self.center_x = self.parent.width - self.parent.arrowimg.width*2
		self.center_y = self.parent.player.center_y + 20
		self.serve()
		self.stat = 0

	def isOut(self):
		if self.x < self.parent.x:
			self.resetpos()

	def main(self):
		self.move()
		self.isOut()
		if self.stat == 0:
			self.center_y = self.parent.player.center_y + 20

class Balloon(Image):
	velocity_x = NumericProperty(0)
	velocity_y = NumericProperty(0)
	velocity = ReferenceListProperty(velocity_x, velocity_y)

	def main(self):
		self.move()
		self.isOut()
		self.isShot()

	def serve(self, dt=0):
		self.velocity = (0, randint(10, 100)/10.0)

	def move(self):
		self.pos = Vector(*self.velocity) + self.pos

	def bounce(self, arrow):
		if self.collide_widget(arrow):
			if (self.parent.ballpopsound.get_pos() > 0):
				self.parent.ballpopsound.stop()
			self.parent.ballpopsound.play()
			return True
		return False

	def resetpos(self):
		temp = randint(1, 1024)
		self.top = self.parent.y
		self.x = (self.parent.width-(4*self.parent.arrowimg.width))*temp/1024
		self.serve()

	def isOut(self):
		if self.y > self.parent.top:
			self.resetpos()

	def isShot(self):
		if(self.bounce(self.parent.arrow)):
			self.parent.score += 1
			self.resetpos()

class ArcheryGame(Widget):
	score = NumericProperty(0)
	player = ObjectProperty(None)
	arrow = ObjectProperty(None)
	balloon = ObjectProperty(None)
	balloon1 = ObjectProperty(None)
	balloon2 = ObjectProperty(None)
	balloon3 = ObjectProperty(None)
	balloon4 = ObjectProperty(None)
	balloon5 = ObjectProperty(None)
	balloon6 = ObjectProperty(None)
	balloon7 = ObjectProperty(None)
	arrowimg = ObjectProperty(None)
	sound = SoundLoader.load('Koala.mp3')
	ballpopsound = SoundLoader.load('Pop.mp3')
	shotsound = SoundLoader.load('Pew.mp3')
	countdown = SoundLoader.load('count.mp3')
	counttime = NumericProperty(120*60)
	peri = PeriBoard()

	def serve(self):
		self.balloon.serve()
		self.balloon1.serve()
		self.balloon2.serve()
		self.balloon3.serve()
		self.balloon4.serve()
		self.balloon5.serve()
		self.balloon6.serve()
		self.balloon7.serve()

	def update(self, dt):
		global temp
		val = self.peri.getLight()*0.2 + 0.8*temp
		temp = val
		self.player.center_y = val
		self.balloon.main()
		self.balloon1.main()
		self.balloon2.main()
		self.balloon3.main()
		self.balloon4.main()
		self.balloon5.main()
		self.balloon6.main()
		self.balloon7.main()
		self.arrow.main()
		self.keypress()
		self.arrowimg.move()
		if (self.counttime > 0):
		    self.counttime -= 1
		if (self.counttime < 600):
			self.countdown.play()
		if (self.counttime == 1):
			self.countdown.stop()
			self.parent.parent.current = "end"
		if (self.counttime == 0):
			self.countdown.stop()
			self.ballpopsound.unload()
			self.shotsound.unload()

	def keypress(self):
		if (self.peri.getSwitch() is True) and (self.arrow.velocity_x == 0):	
			self.arrow.serve(vel=(-8, 0))
			self.arrow.stat = 1
			self.shotsound.play()

class EndScreen(Screen):
	def on_pre_enter(self):
		endsound = SoundLoader.load('gameover.mp3')
		endsound.play()

class GameScreen(Screen):
	def on_pre_enter(self):
		self.archerygame.sound.play()
		self.archerygame.serve()
		Clock.schedule_interval(self.archerygame.update, 1.0 / 60.0)

class OpenScreen(Screen):
	sound = SoundLoader.load('Koala.mp3')

	def on_pre_enter(self):
		self.sound.play()

	def goGame(self):
		self.sound.unload
                self.sound.stop()
		self.parent.current = "game"

class ArcheryApp(App):
	def build(self):
		sm = ScreenManager(transition=FadeTransition())
		sm.add_widget(OpenScreen(name='open'))
		sm.add_widget(GameScreen(name='game'))
		sm.add_widget(EndScreen(name='end'))
		return sm

if __name__ == '__main__':
	ArcheryApp().run()
